#include "SqcApp.h"

void set_title(SqcApp * a)
{
	unsigned int cp = a->m->currentFrame - 1;
	std::string title("");
	if (a->m->images.size()) {
		char *tmpStr = (char *)malloc(FL_PATH_MAX);
		title += fl_filename_name(a->m->files[cp].c_str());
		title += "    ";
		sprintf(tmpStr, "%d", FreeImage_GetWidth(a->m->images[cp]));
		title += tmpStr;
		title += " x ";
		sprintf(tmpStr, "%d", FreeImage_GetHeight(a->m->images[cp]));
		title += tmpStr;
		title += "    ";
		sprintf(tmpStr, "%f", a->w->viewport->zoom);
		title += tmpStr;
		title += "    ";
		delete tmpStr;
	}
	title += "SqCheck";
	a->w->window->label(title.c_str());
	a->w->viewport->redraw();
}

void SqcGlWindow::draw()
{
	SqcApp *a = (SqcApp *) parentApp;
	float width, height;
	width = (float)Fl_Gl_Window::w();
	height = (float)Fl_Gl_Window::h();

	glViewport(0, 0, (int)width, (int)height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, width, 0, height, -1.0, 1.0);
	//glMatrixMode(GL_MODELVIEW);

	glClearColor(0.5f, 0.5f, 0.5f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	int imgW = 256, imgH = 256, dptW = 256, dptH = 256;

	/*
	 * returning if no image is loaded; if at least one is loaded the
	 * system will manage the rest
	 */
	if (!a->m->images.size())
		return;

	int curFrame = (int)a->w->progress->value() - 1;
	imgW = FreeImage_GetWidth(a->m->images[curFrame]);
	imgH = FreeImage_GetHeight(a->m->images[curFrame]);
	dptW = FreeImage_GetWidth(a->m->depths[curFrame]);
	dptH = FreeImage_GetHeight(a->m->depths[curFrame]);

	// if 'fit' is selected we adjust the image on the viewport
	if (a->w->fit->value()) {
		panX = 0;
		panY = 0;
		float rapW = width / imgW;
		float rapH = height / imgH;
		if (rapW < rapH)
			zoom = rapW;
		else
			zoom = rapH;
	}
	// pans with offsets, to report to the middle of the screen
	float offPanX = panX + width / 2;
	float offPanY = panY + height / 2;

	float zoom = a->w->viewport->zoom;
	glRasterPos2f(0, 0);
	glBitmap(0, 0, 0.0f, 0.0f, offPanX - (imgW / 2) * zoom,
		 offPanY - (imgH / 2) * zoom, NULL);
	glPixelZoom(zoom, zoom);

	// getting image display options from the window menu
	int rgbSel = a->w->rgb->value();
	int rgbaSel = a->w->rgba->value();
	int redSel = a->w->red->value();
	int greenSel = a->w->green->value();
	int blueSel = a->w->blue->value();
	int alphaSel = a->w->alpha->value();
	int depthSel = a->w->depth->value();

	FIBITMAP *chn;

	// setting default without blending
	glDisable(GL_BLEND);

	/*
	 * glEnable(GL_BLEND); glBlendFunc (GL_SRC_ALPHA,
	 * GL_ONE_MINUS_SRC_ALPHA); glDrawPixels(imgW, imgH, GL_ALPHA,
	 * GL_UNSIGNED_BYTE, alpha.accessPixels()); glBlendFunc (GL_ONE,
	 * GL_DST_ALPHA);
	 */

	if (rgbaSel) {
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glDrawPixels(imgW, imgH, SQ_COLS_ORDER, GL_UNSIGNED_BYTE,
				FreeImage_GetBits(a->m->images[curFrame]));
	} else if (rgbSel) {
		glDrawPixels(imgW, imgH, SQ_COLS_ORDER, GL_UNSIGNED_BYTE,
				FreeImage_GetBits(a->m->images[curFrame]));
	} else if (redSel) {
		if ((chn = FreeImage_GetChannel(a->m->images[curFrame], FICC_RED)) != NULL)
			glDrawPixels(imgW, imgH, GL_LUMINANCE, GL_UNSIGNED_BYTE,
					FreeImage_GetBits(chn));
		FreeImage_Unload(chn);
	} else if (greenSel) {
		if ((chn = FreeImage_GetChannel(a->m->images[curFrame], FICC_GREEN)) != NULL)
			glDrawPixels(imgW, imgH, GL_LUMINANCE, GL_UNSIGNED_BYTE,
					FreeImage_GetBits(chn));
		FreeImage_Unload(chn);
	} else if (blueSel) {
		if ((chn = FreeImage_GetChannel(a->m->images[curFrame], FICC_BLUE)) != NULL)
			glDrawPixels(imgW, imgH, GL_LUMINANCE, GL_UNSIGNED_BYTE,
					FreeImage_GetBits(chn));
		FreeImage_Unload(chn);
	} else if (alphaSel) {
		if ((chn = FreeImage_GetChannel(a->m->images[curFrame], FICC_ALPHA)) != NULL)
			glDrawPixels(imgW, imgH, GL_LUMINANCE, GL_UNSIGNED_BYTE,
					FreeImage_GetBits(chn));
		FreeImage_Unload(chn);
	} else if (depthSel) {
		glDrawPixels(dptW, dptH, GL_LUMINANCE, GL_UNSIGNED_BYTE,
				FreeImage_GetBits(a->m->depths[curFrame]));
	}

	set_title(a);
	cachedW = width;
	cachedH = height;
}

int SqcGlWindow::handle(int event)
{
	SqcApp *a = (SqcApp *) parentApp;
	int x = Fl::event_x();
	int y = Fl::event_y();
	
	int ret = Fl_Gl_Window::handle(event);
	
	switch (event) {
	case FL_PUSH:
		xMouse = (float)x;
		yMouse = (float)y;
		ret = 1;
		break;
	case FL_DRAG:
		{
			if (Fl::event_state(FL_BUTTON3)) {
				float xMouseDif = x - xMouse;
				float yMouseDif = y - yMouse;
				float zoomRatio = 1;
				zoomRatio += (xMouseDif + yMouseDif) / 1000;

				zoom *= zoomRatio;
				panX *= zoomRatio;
				panY *= zoomRatio;

				if (zoom < 0)
					zoom = 0;
				redraw();
			}
			if (Fl::event_state(FL_BUTTON2)) {
				panX += x - xMouse;
				panY += yMouse - y;
				redraw();
			}
			if (Fl::event_state(FL_BUTTON1)) {
				int fr = a->m->images.size();
				if (fr) {
					float xMouseDif = x - xMouse;
					float limit = (float)w() / fr / 2;
					slow += xMouseDif;
					if (slow > limit) {
						fplay_cb(NULL, a);
						slow = 0;
					} else if (slow < 0 - limit) {
						fback_cb(NULL, a);
						slow = 0;
					}
					redraw();
				}

			}
			xMouse = (float)x;
			yMouse = (float)y;
		}
		ret = 1;
		break;
	case FL_DND_ENTER:
		a->w->viewport->size(a->w->viewportGrp->w() - 4, a->w->viewportGrp->h() - 4);
		a->w->viewport->position(a->w->viewportGrp->x() + 2, a->w->viewportGrp->y() + 2);
		a->w->viewportGrp->box(FL_DOWN_BOX);
		a->w->viewportGrp->redraw();
		ret = 1;
		break;
	case FL_DND_LEAVE:
		a->w->viewport->position(a->w->viewportGrp->x(), a->w->viewportGrp->y());
		a->w->viewport->size(a->w->viewportGrp->w(), a->w->viewportGrp->h());
		a->w->viewportGrp->box(FL_FLAT_BOX);
		a->w->viewportGrp->redraw();
		ret = 1;
		break;
	case FL_DND_DRAG:
		ret = 1;
		break;
	case FL_DND_RELEASE:
		a->w->viewport->position(a->w->viewportGrp->x(), a->w->viewportGrp->y());
		a->w->viewport->size(a->w->viewportGrp->w(), a->w->viewportGrp->h());
		a->w->viewportGrp->box(FL_FLAT_BOX);
		a->w->viewportGrp->redraw();
		ret = 1;
		break;
	case FL_PASTE:
		a->append_images_dnd(Fl::event_length(), Fl::event_text());
		ret = 1;
		break;
	}
	return ret;
	
}

void fit_cb(void *cb, SqcApp * a)
{
	a->w->viewport->redraw();
}

void orig_cb(void *cb, SqcApp * a)
{
	a->w->fit->clear();
	a->w->viewport->zoom = 1;
	a->w->viewport->panX = 0;
	a->w->viewport->panY = 0;
	a->w->viewport->redraw();
}

void progress_cb(void *cb, SqcApp * a)
{
	a->m->currentFrame = (unsigned int)a->w->progress->value();
	if (a->m->images.size() >= a->m->currentFrame)
		set_title(a);
}

void begin_cb(void *cb, SqcApp * a)
{
	a->w->progress->value(a->w->progress->minimum());
	a->w->progress->do_callback();
}

void fback_cb(void *cb, SqcApp * a)
{
	if (a->w->progress->value() == a->w->progress->minimum())
		a->w->progress->value(a->w->progress->maximum() + 1);
	a->w->progress->value(a->w->progress->value() - 1);
	a->w->progress->do_callback();
}

void back_timer(void *app)
{
	SqcApp *a = (SqcApp *) app;
	double interval = 1.0 / a->w->fps->value();
	fback_cb(NULL, a);
	Fl::repeat_timeout(interval, &back_timer, app);
}

void fplay_cb(void *cb, SqcApp * a)
{
	if (a->w->progress->value() == a->w->progress->maximum())
		a->w->progress->value(a->w->progress->minimum() - 1);
	a->w->progress->value(a->w->progress->value() + 1);
	a->w->progress->do_callback();
}

void play_timer(void *app)
{
	SqcApp *a = (SqcApp *) app;
	double interval = 1.0 / a->w->fps->value();
	fplay_cb(NULL, a);
	Fl::repeat_timeout(interval, &play_timer, app);
}

void back_cb(void *cb, SqcApp * a)
{
	double interval = 1.0 / a->w->fps->value();
	a->w->play->clear();
	if (a->w->back->value()) {
		Fl::remove_timeout(&play_timer, a);
		Fl::add_timeout(interval, &back_timer, a);
	} else
		Fl::remove_timeout(&back_timer, a);
}

void play_cb(void *cb, SqcApp * a)
{
	double interval = 1.0 / a->w->fps->value();
	a->w->back->clear();
	if (a->w->play->value()) {
		Fl::remove_timeout(&back_timer, a);
		Fl::add_timeout(interval, &play_timer, a);
	} else
		Fl::remove_timeout(&play_timer, a);
}

void endb_cb(void *cb, SqcApp * a)
{
	a->w->progress->value(a->w->progress->maximum());
	a->w->progress->do_callback();
}

void SqcApp::append_images(int argc, char **argv)
{
	int i;
	double firstNewFrame = 0;
	// string for the first sharp character found
	std::string firstArgStr(argv[1]);
	// string for the last sharp character found
	std::string lastArgStr(argv[1]);
	std::string::size_type firstSharp = firstArgStr.find_first_of('#');
	// if no sharp found, load arguments as files
	if (firstSharp == string::npos) {

		firstNewFrame = m->files.size() + 1;
		for (i = 1; i < argc; i++)
			m->append_image(argv[i], m->resScale);
	} else if (argc > 2) {
		std::string::size_type lastSharp = lastArgStr.find_last_of('#');

		// verifying/parsing the sequence range argument, the second one
		std::string secondArgStr(argv[2]);
		vector < string > range = tokenize(&secondArgStr, "-");
		int first, last;

		if (!sscanf(range[0].c_str(), "%d", &first)) {
			std::cout << "Error: first range argument is invalid: "
			    << range[0] << "." << std::endl;
			return;
		}
		if (!sscanf(range[1].c_str(), "%d", &last)) {
			std::
			    cout << "Error: last range argument is invalid: " <<
			    range[1] << "." << std::endl;
			return;
		}
		if (first > last) {
			std::cout <<
			    "Error: first range argument greater than the last: "
			    << range[0] << " > " << range[1] << "." << std::
			    endl;
			return;
		}
		// length of the sharp character sequence
		unsigned int sharpLen = (lastSharp - firstSharp) + 1;

		firstNewFrame = m->files.size() + 1;

		char buffer[256];
		sprintf(buffer, "%d", sharpLen);
		std::string format("%.");
		format.append(buffer);
		format.append("d");

		for (i = first; i <= last; i++) {
			char number[256];
			sprintf(number, format.c_str(), i);
			string fileName(argv[1]);
			fileName.replace(firstSharp, sharpLen, number);
			m->append_image(fileName.c_str(), m->resScale);
		}
	}

	w->progress->maximum(m->images.size());
	w->progress->value(firstNewFrame);
	w->progress->redraw();
	w->progress->do_callback();
}

void SqcApp::append_images_dnd(int dndlen, const char *dndbuf)
{
	const char *end, *start, *fin, *curs;
	fin = dndbuf + dndlen;
	
	int filesno = 1;
	
	for (end = dndbuf; end <= fin; end++) {
		if (*end == '\n')
			filesno++;
	}
	//now decrease the number if we have a last newline
	if (*(fin - 2) == '\n' || *(fin - 1) == '\n')
		filesno--;
	
	double firstNewFrame = m->files.size() + 1;
	
	char **fnames = (char **)malloc(filesno * sizeof(char *));
	memset(fnames, 0, sizeof(fnames));
	char fnamebuf[4096] = "";
	
	int i;
	end = dndbuf;
	start = dndbuf;
	for (i = 0; i < filesno; i++) {
		//setting the start pointer in name
		if (*start == 0) {
			break;
		} else if (strncmp(start, "file:///", 8) == 0)
			start += 7;
		//setting cursor to start
		curs = start;
		
		while (*curs != '\n' && *curs != 0 && curs != fin) {
			curs++;
		}
		end = curs;
		if (*(end - 1) == '\n')
			end--;
		if (*(end - 1) == '\r')
			end--;
		/* do stuff here */
		if (end >= start) {
			strncpy(fnamebuf, start, end - start);
			fnamebuf[end - start] = 0;
			fnames[i] = strdup(fnamebuf);
		}
		
		start = curs + 1;
	}
	
	//sorting and loading; using the char pointers differently
	i = filesno;
	int j = 0;
	int curid = 0;
	while (i > 0) {
		for (j = 0; j < filesno; j++) {
			if (fnames[curid] == NULL) {
				curid = j;
				continue;
			}
			
			if (fnames[j] != NULL && strcmp(fnames[curid], fnames[j]) >= 1) {
				curid = j;
			}
		}
		
		if (fnames[curid] != NULL) {
			m->append_image(fnames[curid], m->resScale);
			fnames[curid] = NULL;
		}
		
		i--;
	}
	
	free(fnames);
	
	w->progress->maximum(m->images.size());
	w->progress->value(firstNewFrame);
	w->progress->redraw();
	w->progress->do_callback();
}

#if FL_MAJOR_VERSION==1
#if FL_MINOR_VERSION==3
void append_browser_cb(void *cb, SqcApp * a)
{
	std::vector < std::string > lastDir(get_option("lastDir"));
	std::string dir("");
	unsigned toksNr;
	for (toksNr = 1; toksNr < lastDir.size(); toksNr++)
		dir += lastDir[toksNr];
	Fl_Native_File_Chooser *lfc =
	    new Fl_Native_File_Chooser(Fl_Native_File_Chooser::BROWSE_MULTI_FILE);
	lfc->directory(dir.c_str());
	lfc->filter(NULL);
	lfc->type(Fl_Native_File_Chooser::BROWSE_MULTI_FILE);
	lfc->title("Load Images...");
	lfc->show();
	if (!lfc->count())
		return;
	int i;
	double firstNewFrame = a->m->files.size() + 1;
	for (i = 0; i < lfc->count(); i++)
		a->m->append_image(lfc->filename(i), a->m->resScale);
	lastDir.clear();
	lastDir.push_back("lastDir");
	dir = lfc->filename(0);
	lastDir.push_back(dir.substr(0, dir.find_last_of('/')));
	set_option(lastDir);
	delete lfc;
	a->w->progress->maximum(a->m->images.size());
	a->w->progress->value(firstNewFrame);
	a->w->progress->redraw();
	a->w->progress->do_callback();
}

#else
void append_browser_cb(void *cb, SqcApp * a)
{
	std::vector < std::string > lastDir(get_option("lastDir"));
	std::string dir("");
	unsigned toksNr;
	for (toksNr = 1; toksNr < lastDir.size(); toksNr++)
		dir += lastDir[toksNr] + " ";
	Fl_File_Chooser *lfc =
	    new Fl_File_Chooser(dir.c_str(), "*", 1, "Load Images...");
	lfc->show();
	while (lfc->visible())
		Fl::wait();
	if (!lfc->count())
		return;
	int i;
	double firstNewFrame = a->m->files.size() + 1;
	for (i = 1; i < lfc->count() + 1; i++)
		a->m->append_image(lfc->value(i), a->m->resScale);
	lastDir.clear();
	lastDir.push_back("lastDir");
	dir = lfc->directory();
	dir += "/";
	lastDir.push_back(dir);
	set_option(lastDir);
	delete lfc;
	a->w->progress->maximum(a->m->images.size());
	a->w->progress->value(firstNewFrame);
	a->w->progress->redraw();
	a->w->progress->do_callback();
}

#endif //FL_MINOR_VERSION
#endif //FL_MAJOR_VERSION

void set_res_scale_cb(void *cb, SqcApp * a)
{
	std::vector < std::string > resScale;
	resScale.clear();
	resScale.push_back("resScale");
	
	if (a->w->fullRes->value() != 0) {
		a->m->resScale = 1.0;
		resScale.push_back("1");
	} else if (a->w->threeQuarters->value() != 0) {
		a->m->resScale = 3.0 / 4.0;
		resScale.push_back("3/4");
	} else if (a->w->twoThirds->value() != 0) {
		a->m->resScale = 2.0 / 3.0;
		resScale.push_back("2/3");
	} else if (a->w->half->value() != 0) {
		a->m->resScale = 1.0 / 2.0;
		resScale.push_back("1/2");
	} else if (a->w->third->value() != 0) {
		a->m->resScale = 1.0 / 3.0;
		resScale.push_back("1/3");
	} else if (a->w->quarter->value() != 0) {
		a->m->resScale = 1.0 / 4.0;
		resScale.push_back("1/4");
	} else {
		a->m->resScale = 1.0;
		resScale.push_back("1");
	}
	
	set_option(resScale);
}

void clear_images_cb(void *cb, SqcApp * a)
{
	a->m->clear_images();
	a->w->progress->value(1);
	a->w->progress->maximum(1);
	a->w->progress->redraw();
	set_title(a);
}
