#include "SqcConfig.h"
#include <cstdlib>
#include <iostream>

#ifdef _WIN32
#define FILE_SEPARATOR "\\"
#define HOME_VAR "HOMEPATH"
#else
#define FILE_SEPARATOR "/"
#define HOME_VAR "HOME"
#endif

string getConfigFile()
{
	char *homeDir = getenv(HOME_VAR);
	string configFile(homeDir);
	configFile.append(FILE_SEPARATOR);
	configFile.append(".SqCheck.conf");

	return configFile;
}

// { copy/paste from
// http://www.oopweb.com/CPP/Documents/CPPHOWTO/Volume/C++Programming-HOWTO-7.html
// returns a a vector with the tokens contained in the argument string,
// splitting it
// by the 'delimiters' argument

vector < string > tokenize(const string * str, const char *delimiters = " ")
{
	vector < string > tokens;
	string::size_type lastPos = str->find_first_not_of(delimiters, 0);
	string::size_type pos = str->find_first_of(delimiters, lastPos);
	while (string::npos != pos || string::npos != lastPos) {
		tokens.push_back(str->substr(lastPos, pos - lastPos));
		lastPos = str->find_first_not_of(delimiters, pos);
		pos = str->find_first_of(delimiters, lastPos);
	}
	return tokens;
}

// }

// returns a string made by concatenating the strings vector argument,
// with 'delimiters' in-between. 

string concat(const vector < string > *strArray, const char *delimiters = " ")
{
	string cat("");
	unsigned int i;
	for (i = 0; i < (strArray->size() - 1); i++) {
		cat += (*strArray)[i];
		cat += delimiters;
	}
	// stop here, we don't want to have a delimiter appended to the end
	cat += (*strArray)[i];
	return cat;
}

vector < string > read_file()
{
	string pathStr = getConfigFile();
	const char *path = pathStr.c_str();

	vector < string > lines;
	string line;
	ifstream ifile(path);
	while (getline(ifile, line))
		lines.push_back(line);

	return lines;
}

bool write_file(vector < string > lines)
{
	string pathStr = getConfigFile();
	const char *path = pathStr.c_str();

	ofstream ofile(path);
	unsigned int i;
	for (i = 0; i < lines.size(); i++)
		ofile << lines[i] << endl;
	ofile.close();

	return true;
}

vector < string > get_option(const char *optionName)
{

	vector < string > value;
	value.push_back("");
	vector < string > lines(read_file());
	if (!lines.size()) {
		value.clear();
		return value;
	}
	unsigned int i;
	for (i = 0; i < lines.size(); i++) {
		value = tokenize(&lines[i]);
		if (!value[0].compare(optionName))
			return value;
	}
	value.clear();

	return value;
}

bool set_option(vector < string > option)
{
	vector < string > value;
	value.push_back("");
	vector < string > lines(read_file());
	unsigned int i;
	for (i = 0; i < lines.size(); i++) {
		value = tokenize(&lines[i]);
		if (!value[0].compare(option[0].c_str())) {
			lines[i] = concat(&option);
			write_file(lines);
			return true;
		}
	}
	lines.push_back(concat(&option));
	write_file(lines);

	return false;
}
