#include "SqcManager.h"
#include <cfloat>
#include <cstring>
#include <cstdio>

typedef struct _Pixel {
	BYTE r;
	BYTE g;
	BYTE b;
	BYTE a;
} Pixel;

static void swap_r_with_b(Pixel * pixelData, Pixel * imgData, const size_t len)
{
	size_t i;
	for (i = 0; i < len; i++) {
		BYTE tmpR = pixelData[i].r;
		imgData[i].r = pixelData[i].b;
		imgData[i].g = pixelData[i].g;
		imgData[i].b = tmpR;	// n-am nevoie de temp decat daca o sa fie 
		// 
		// de bagat swapul shi in in non-iff
		imgData[i].a = pixelData[i].a;
	}
}

SqcManager::SqcManager()
{
	resScale = 1.0;
	currentFrame = 0;
	currentTime = 0.0;
}

SqcManager::~SqcManager()
{
	clear_images();
}

void SqcManager::clear_images()
{
	unsigned int i;
	
	for (i = 0; i < images.size(); i++) {
		FreeImage_Unload(images[i]);
	}
	for (i = 0; i < depths.size(); i++) {
		FreeImage_Unload(depths[i]);
	}
	
	images.clear();
	depths.clear();
	files.clear();
}

static FIBITMAP *load_image(const char* fileName)
{
	FREE_IMAGE_FORMAT fif = FreeImage_GetFileType(fileName, 0);
	FIBITMAP *dib = NULL;
	
	if (fif == FIF_UNKNOWN)
		fif = FreeImage_GetFIFFromFilename(fileName);
	
	if ((fif != FIF_UNKNOWN) && FreeImage_FIFSupportsReading(fif)) {
		dib = FreeImage_Load(fif, fileName, BMP_DEFAULT);
	} 
	
	if (dib == NULL) {
		dib = FreeImage_Allocate(0, 0, 32, 0, 0, 0);
	}
	
	return dib;
}

bool SqcManager::append_image(const char *path, const float scale)
{
	FILE *f = fopen(path, "r");
	if (f == 0) {
		return true;
	} else {
		fclose(f);
	}
	
	FIBITMAP *img = NULL;
	FIBITMAP *dpt = NULL;
	FIBITMAP *tmp = NULL;
	
	// if image is iff, we create a temporary iff_image
	iff_image *iffImg = iff_load(path);
	if (iffImg) {
		size_t size = iffImg->width * iffImg->height;
		
		if (iffImg->rgba) {
			img = FreeImage_Allocate(iffImg->width, iffImg->height, 32, 0, 0, 0);
			swap_r_with_b((Pixel *) iffImg->rgba,
				      (Pixel *) FreeImage_GetBits(img), size);
		}
		
		if (iffImg->zbuffer) {
			//std::cout << "Near: " << iffImg->znear << std::endl;
			//std::cout << "Far : " << iffImg->zfar << std::endl;
			size_t i;
			char *tmpDpt = new char[size];
			for (i = 0; i < size; i++) {
				tmpDpt[i] = (unsigned char)(255.0 *
						(0 -
						(iffImg->zbuffer[i + 1])));
			}
			dpt = FreeImage_Allocate(iffImg->width, iffImg->height, 8, 0, 0, 0);
			memcpy(FreeImage_GetBits(dpt), tmpDpt,
			       iffImg->width * iffImg->height);
			delete tmpDpt;
		}
		iff_free(iffImg);
	}

	/*
	 * Image was not an IFF type, trying to load with FreeImage.
	 */
	else if ((img = load_image(path)) != 0) {
		if (FreeImage_GetImageType(img) != FIT_BITMAP) {
			tmp = FreeImage_ConvertToType(img, FIT_BITMAP, 1);
			FreeImage_Unload(img);
			img = tmp;
		}
		if (FreeImage_GetBPP(img) != 32) {
			tmp = FreeImage_ConvertTo32Bits(img);
			FreeImage_Unload(img);
			img = tmp;
		}
	} else {
		return true;
	}

	/*
	 * Size issues: width, height and scaling
	 */
	int width = FreeImage_GetWidth(img);
	int height = FreeImage_GetHeight(img);
	if (scale != 1.0) {
		width = (int)(width * scale);
		height = (int)(height * scale);
		if (FreeImage_HasPixels(img) == true) {
			tmp = FreeImage_Rescale(img, width, height, FILTER_BOX);
			FreeImage_Unload(img);
			img = tmp;
		}
		if (FreeImage_HasPixels(dpt) == true)
		{
			tmp = FreeImage_Rescale(dpt, width, height, FILTER_BOX);
			FreeImage_Unload(dpt);
			dpt = tmp;
		}
	}

	/*
	 * storing the image pointers
	 */
	images.push_back(img);
	depths.push_back(dpt);
	files.push_back(path);
	
	return true;
}
